var fs = require('fs');
var q = require('q');
var util = require('util');
var path = require('path');
var cp = require('child_process');
var futil = require('flapjack-util');
var events = require('events');

function GulpProject(src) {
  src = path.resolve(src);
  if(!src || !fs.existsSync(src) || !fs.statSync(src).isDirectory()) {
    throw util.format('Invalid directory: "%s"', src.magenta);
  }
  this.src = src;
  var pathsPath = path.resolve(this.src, 'paths.js');
  var paths;
  if(fs.existsSync(pathsPath) && fs.statSync(pathsPath).isFile()) {
    try {
      paths = require(pathsPath);
    } catch(e) {
      throw util.format('Invalid paths file: "%s"', pathsPath);
    }
  }
  this.paths = new futil.Paths(this.src, paths).validate();
}

util.inherits(GulpProject, events.EventEmitter);

GulpProject.prototype.status = function() {
  var status = GulpProject.Status.LAST_BUILD_SUCCESSFUL;
  if(this.building && this.building.promise && this.building.promise.isPending()) {
    status = GulpProject.Status.BUILDING;
  } else if(this.error) {
    status = GulpProject.Status.LAST_BUILD_FAILED;
  }
  return status;
};

GulpProject.prototype.build = function() {
  if(this.status() !== GulpProject.Status.BUILDING) {
    this.emit(GulpProject.Event.BUILDING, this);

    this.building = q.defer();
    this.error = undefined;

    var p = cp.fork(path.resolve(__dirname, 'gulp-runner.js'), { cwd: this.src });

    p.on('message', function(m) {
      var message = new futil.GulpMessage(m);
      if(message.data === 'complete' && !this.error) {
        this.emit(GulpProject.Event.BUILD_SUCCESSFUL, this);
        this.building.resolve(this);
      } else if(message.type === futil.GulpMessage.Type.ERROR) {
        this.emit(GulpProject.Event.BUILD_FAILED, this.error);
        this.error = message.data;
        this.building.reject(this.error);
      }
    }.bind(this));

    p.send(new futil.GulpMessage({
        type: futil.GulpMessage.Type.MESSAGE,
        data: 'start'
      }));
  }
  return this.building.promise;
};

GulpProject.Event = {
  BUILDING: 'building',
  BUILD_SUCCESSFUL: 'build-successful',
  BUILD_FAILED: 'build-failed'
};
Object.freeze(GulpProject.Event);

GulpProject.Status = {
  BUILDING: 'building',
  LAST_BUILD_SUCCESSFUL: 'last-build-successful',
  LAST_BUILD_FAILED: 'last-build-failed'
};
Object.freeze(GulpProject.Status);

module.exports = GulpProject;
